
local Class = require 'libs.hump.class'
local Entity = require 'entities.Entity'
local HC = require 'libs.HC'

local player = Class{
  __includes = Entity
}


local text = {}
function player:init(x, y, explosion)
  -- Properties
  self.inControl = true
  self.isAlive = true
  self.frame_width = 87
  self.frame_height = 72
  self.xOriginOffset = self.frame_width / 2
  self.yOriginOffset = self.frame_height / 2
  self.x = x
  self.y = y
  self.rot = 0
  self.maxRot = .2
  self.rotSpeed = 5
  self.rotTolerance = .1
  self.scale = .5
  self.moveSpeed = 300
  
  -- Animation Info
  self.frames = {}
  table.insert(self.frames, love.graphics.newQuad(330, 1371, self.frame_width, self.frame_height, sheetWidth, sheetHeight))
  table.insert(self.frames, love.graphics.newQuad(372, 1132, self.frame_width, self.frame_height, sheetWidth, sheetHeight))
  table.insert(self.frames, love.graphics.newQuad(222, 1562, self.frame_width, self.frame_height, sheetWidth, sheetHeight))
  table.insert(self.frames, love.graphics.newQuad(372, 1132, self.frame_width, self.frame_height, sheetWidth, sheetHeight))
  self.currentFrame = 1
  self.animationSpeed = 8
  
  -- Physics Info
  self.rect = HC.rectangle(self.x,self.y,self.frame_width * self.scale ,self.frame_height * self.scale)
  
  -- Particle System
  local img = love.graphics.newImage("assets/whitePuff06.png")
  self.exhaust = love.graphics.newParticleSystem(img, 32)
  self.exhaust:setParticleLifetime(1,2)
  self.exhaust:setSpeed(50,75)
  self.exhaust:setRotation(0,3)
  self.exhaust:setSizes(0.01,0.05)
  self.exhaust:setColors(255,255,255,255,255,255,255,0)
  self.exhaust:setDirection(3)
  self.exhaust:setParticleLifetime( 0.25, .5 )
  self.emitSpeed = .25
  self.emitTimer = self.emitSpeed
  
  self.explosion = explosion
  
  -- Init Class
  Entity.init(self, x, y, self.frame_width, self.frame_height)
end


function player:update(dt)
  
  -- Animation
  self.currentFrame = self.currentFrame + self.animationSpeed * dt
  if self.currentFrame >= table.getn(self.frames)+1 then
    self.currentFrame = 1
  end
  
  -- Movement
  if self.inControl then
    if love.keyboard.isDown("left", "a") and self.x > math.ceil(self.xOriginOffset * self.scale) then
      self.x = self.x - self.moveSpeed * dt
    elseif love.keyboard.isDown("right", "d") and self.x < love.graphics.getWidth( ) - math.ceil(self.xOriginOffset * self.scale) then
      self.x = self.x + self.moveSpeed * dt
    end
    
    if love.keyboard.isDown("up", "w") and self.y > math.ceil(self.yOriginOffset * self.scale) then
      self.y = self.y - self.moveSpeed * dt
      if self.rot > -self.maxRot then
        self.rot = self.rot - dt * self.rotSpeed
      end
    elseif love.keyboard.isDown("down", "s") and self.y < love.graphics.getHeight( ) - math.ceil(self.yOriginOffset * self.scale) then
      self.y = self.y + self.moveSpeed * dt
      if self.rot < self.maxRot then
        self.rot = self.rot + dt * self.rotSpeed
      end
    elseif self.rot ~= 0 then
      self.rot = self.rot + dt * -sign(self.rot)
      if math.abs(self.rot) < self.rotTolerance then
        self.rot = 0
      end
    end
  end
  --Update Physics
  self.rect:moveTo(self.x, self.y)
  
  --Test Collisions
  for shape, delta in pairs(HC.collisions(self.rect)) do
      text[#text+1] = string.format("Colliding. Separating vector = (%s,%s)",delta.x, delta.y)
      self.explosion:boom(self.x,self.y)
  end
    
  while #text > 40 do
    table.remove(text, 1)
  end
  
  -- Update Particles
  self.exhaust:update(dt)
  self.exhaust:moveTo(self.x, self.y)
  self.emitTimer = self.emitTimer + dt
  if self.emitTimer >= self.emitSpeed then
    self.exhaust:emit(5)
    self.emitTimer = 0
  end
end




function player:draw()
  if self.isAlive then
    love.graphics.draw(self.exhaust)
    
    love.graphics.draw(spritesheet, self.frames[math.floor(self.currentFrame)], self.x, self.y, self.rot, self.scale, self.scale, self.xOriginOffset, self.yOriginOffset)
  --  love.graphics.setColor(255,0,0)
  --  self.rect:draw('line')
    for i = 1,#text do
      love.graphics.setColor(0,0,0, 255 - (i-1) * 6)
      love.graphics.print(text[#text - (i-1)], 10, i * 15)
    end
    love.graphics.setColor(255,255,255)
  end
end

function sign(x)
  if x<0 then
    return -1
  else
    return 1
  end
end

return player
